#!/usr/bin/perl
#
# $Revision: 1.2 $
# Copyright 2013 Teleflora
#
# Revert to a set of edir *.tel files.
#

use strict;
use warnings;
use Getopt::Long;
use POSIX;
use English qw( -no_match_vars );
use File::Basename;

our $VERSION = '1.14';

my $CVS_REVISION = '$Revision: 1.2 $';
my $PROGNAME = basename($PROGRAM_NAME);

#######################################
#########   D E F A U L T S   #########
#######################################

my $EMPTY = q{};

my $DAISY_FILESYS_DEF    = "/d";
my $DAISY_DIR_DEF        = $DAISY_FILESYS_DEF . "/daisy";
my $EDIR_BASEDIR_DEF     = $DAISY_FILESYS_DEF . "/edirectories";
my $LOGFILE_DEF          = $DAISY_DIR_DEF     . "/log/edir_revert.log";
my $DAISY_BACKUP_DIR     = "/backups";

############################################
#########   E X I T  S T A T U S   #########
############################################

my $EXIT_OK = 0;
my $EXIT_COMMAND_LINE = 1;
my $EXIT_MUST_BE_ROOT = 2;
my $EXIT_REVERT_DATE_FORMAT = 10;
my $EXIT_REVERT_UNAVAILABLE = 11;
my $EXIT_REVERT_ERROR = 12;

#####################################
#########   G L O B A L S   #########
#####################################

my $DAISY = 1;
my $DAISY_DIR = $DAISY_DIR_DEF;
my $LOGFILE = $LOGFILE_DEF;
my $LOG_STDERR = 0;
my $DEBUG = 0;
my $TIMESTAMP = strftime("%Y%m%d%H%M%S", localtime());
my $ALTROOT = $EMPTY;   # alternate root for testing
my $ALTOS = $EMPTY;     # alternate os for testing

##############################################
#########   C O M M A N D  L I N E   #########
##############################################

my $REVISION = 0;
my $HELP = 0;
my $VERBOSE = 0;
my $REVERT_PREVIOUS = 0;
my $REVERT_DATE = $EMPTY;

GetOptions(
	"version" => \$REVISION,
	"help" => \$HELP,
	"verbose" => \$VERBOSE,
	"previous|revert-previous" => \$REVERT_PREVIOUS,
	"date|revert-date=s" => \$REVERT_DATE,
	"altroot=s" => \$ALTROOT,
	"altos=s" => \$ALTOS,
) || die "Error: invalid command line option, exiting...\n";


# --version
if ($REVISION) {
    print("$PROGNAME $CVS_REVISION\n");
    exit($EXIT_OK);
}

# --help
if ($HELP) {
    print("$PROGNAME $CVS_REVISION\n");
    usage();
    exit($EXIT_OK);
}

#
# Don't accept any command line args... usage() and exit.
#
if ($#ARGV >= 0) {
    usage();
    exit($EXIT_COMMAND_LINE);
}

if ($REVERT_PREVIOUS && $REVERT_DATE ne $EMPTY) {
    print "choose one: --previous or --date=YYYYMMDDHHMMSS\n";
    usage();
    exit($EXIT_COMMAND_LINE);
}

#
# Kick off the real action.
#
my $exit_status = main();

#
# Done - inform system what happened.
#
exit($exit_status);


###############################
#########   M A I N   #########
###############################

sub main
{
    my $rc = $EXIT_OK;

    loginfo("$PROGNAME $CVS_REVISION");

    #
    # Gotta be root or exit.
    #
    if ($EUID != 0) {
	usage();
	logerror("Error: You need root privileges to run this script");
	return($EXIT_MUST_BE_ROOT);
    }

    my $revert_to_date = $EMPTY;
    if ($REVERT_PREVIOUS || $REVERT_DATE eq $EMPTY) {
	$revert_to_date = get_previous_revert_date();
	if ($revert_to_date eq $EMPTY) {
	    logerror("there are no previous tel sets to revert to");
	    $rc = $EXIT_REVERT_UNAVAILABLE;
	}
    }
    elsif ($REVERT_DATE ne $EMPTY) {
	if ($REVERT_DATE =~ /\d{14}/x) {
	    $revert_to_date = get_specific_revert_date($REVERT_DATE);
	    if ($revert_to_date eq $EMPTY) {
		logerror("there is no previous tel set with date: $REVERT_DATE");
		$rc = $EXIT_REVERT_UNAVAILABLE;
	    }
	}
	else {
	    logerror("revert date format: YYYYMMDDHHMMSS");
	    $revert_to_date = $EMPTY;
	    $rc = $EXIT_REVERT_DATE_FORMAT;
	}
    }
    if ($revert_to_date eq $EMPTY) {
	return($rc);
    }

    loginfo("Reverting to previous tel set date: $revert_to_date");
    $rc = edir_revert($revert_to_date);
    if ($rc == $EXIT_OK) {
	loginfo("edir revert successful");
    }
    else {
	logerror("revert of edir returned status: $rc");
    }

    return($rc);
}


#############################################
#########   A P P L I C A T I O N   #########
#############################################

sub edir_revert
{
    my ($revert_to_date) = @_;

    my @db_dirs = get_daisy_db_dirs();

    my $rc = $EXIT_OK;

    foreach my $db_dir (@db_dirs) {
	if (er_revert_db_dir($db_dir, $revert_to_date)) {
	    $rc = $EXIT_REVERT_ERROR;
	}
    }

    return($rc);
}


#
# For the given daisy db dir and revert to date, revert that db dir
# to that tel set date.
#
# Returns
#   1 on success
#   0 if not
#
sub er_revert_db_dir
{
    my ($db_dir, $revert_to_date) = @_;

    my $rc = 1;

    my $tel_set_control_file = "/control.tel.prepatch." . $revert_to_date;
    my $tel_set_dir = $db_dir . $DAISY_BACKUP_DIR;
    my $tel_set_control_path = $tel_set_dir . $tel_set_control_file;
    if (-f $tel_set_control_path) {
	if (er_restore_tel_set($db_dir, $revert_to_date)) {
	    loginfo("tel set $revert_to_date reverted for Daisy db: $db_dir");
	}
	else {
	    logerror("tel set $revert_to_date revert failed for: $db_dir ");
	    $rc = 0;
	}
    }
    else {
	logerror("tel set $revert_to_date does not exist for: $db_dir ");
	$rc = 0;
    }

    return($rc);
}


sub er_restore_tel_set
{
    my ($db_dir, $revert_to_date) = @_;

    my $rc = 1;

    if (-d $db_dir) {
	my $tel_set_files = "/*.tel.prepatch." . $revert_to_date;
	my $tel_set_dir = $db_dir . $DAISY_BACKUP_DIR;
	my $tel_set_glob_expr = $tel_set_dir . $tel_set_files;

	my @tel_set_paths = glob($tel_set_glob_expr);

	foreach my $src_path (@tel_set_paths) {
	    my $src_filename = basename($src_path);
	    if ($src_filename =~ /(.*)\.prepatch\.$revert_to_date/x) {
		my $dst_filename = $1;
		system("cp $src_path $db_dir/$dst_filename");
		system("chown tfsupport:daisy $db_dir/$dst_filename");
		system("chmod 660 $db_dir/$dst_filename");
	    }
	}
	loginfo("tel set $revert_to_date restored for: $db_dir");
    }
    else {
	logerror("daisy db dir does not exist: $db_dir");
	$rc = 0;
    }

    return($rc);
}


#########################################
#########   F U N C T I O N S   #########
#########################################


sub usage
{
    print("Usage:\n");
    print("$PROGNAME --version\n");
    print("$PROGNAME --help\n");
    print("$PROGNAME [--verbose] [--previous|--date=YYYYMMDDHHMMSS]\n");
    print("\n");

    return(1);
}


sub loginfo
{
    my ($message) = @_;

    chomp($message);
    $message = "<I> $message";
    logit($message);

    return(1);
}


sub logerror
{
    my ($message) = @_;

    chomp($message);
    $message = "<E> $message";
    logit($message);

    return(1);
}


sub logit
{
    my ($message) = @_;

    my $logtime = strftime("%Y-%m-%d %H:%M:%S", localtime());

    chomp($message);
    if (open(my $logfh, '>>', $LOGFILE)) {
	print($logfh "$logtime ($PROGNAME-$PID) $message\n");
	close($logfh);
    }
    else {
	print "$logtime ($PROGNAME-$PID) $message\n";
    }

    if ($VERBOSE) {
	print $message . "\n";
    }

    return(1);
}


#
# Subroutine to run system commands.  This logs an entry.
#
sub mysystem
{
    my ($command) = @_;

    my $returnval = $EMPTY;
    my $log_file = $LOGFILE;

    my $stderr_dest = ($LOG_STDERR) ? "$log_file" : "/dev/null";

    system("$command >> $log_file 2>> $stderr_dest");
    $returnval = WEXITSTATUS($CHILD_ERROR);
    loginfo("$command Returned: $returnval");

    return ($returnval)
}


#
# Return a list of verified POS dirs.  For Daisy, remember, there can be
# more than one daisy database dir!
#
sub get_daisy_db_dirs
{
    my @daisy_dirs = ();
    my $daisy_top_dir = '/d';

    # Daisy POS can have one or more database dirs
    my @daisy_db_dirs = glob("$daisy_top_dir/*");
    for my $daisy_db_dir (@daisy_db_dirs) {

	# must be a directory
	next if (! -d $daisy_db_dir);

	# skip old daisy dirs
	next if ($daisy_db_dir =~ /.+-\d{12}$/x);

	# must contain the magic files
	next if (! -e "$daisy_db_dir/flordat.tel");
	next if (! -e "$daisy_db_dir/control.dsy");

	# must be daisy 8.0+
	next if (! -d "$daisy_db_dir/bin");

	push(@daisy_dirs, $daisy_db_dir);
    }

    return(@daisy_dirs);
}


#
# Count the number of tel sets in a given "backups" dir.
# Assumption: There is one "control.tel.prepatch.$timestamp" file
# per set of backed up tel files, so just count them to determine
# the number of backed up tel sets.
#
# Returns
#   number of sets
#
sub get_tel_set_count
{
    my ($db_dir) = @_;

    my @control_files = glob("$db_dir/backups/control.tel.prepatch.*");

    my $tel_set_count = scalar(@control_files);

    return($tel_set_count);
}


#
# Get a list of the tel set dates from the default Daisy database dir.
#
# Returns the number of tel sets
#
sub get_tel_set_dates
{
    my ($backup_dir, $timestamps) = @_;

    @{$timestamps} = ();

    my @control_files = glob("$backup_dir/control.tel.prepatch.*");
    if (@control_files) {
	# make a list of timestamps from each tel set
	foreach my $filename (@control_files) {
	    if ($filename =~ /.*\.tel\.prepatch\.(\d{14})/x) {
		push(@{$timestamps}, $1);
	    }
	}
    }

    return(scalar(@{$timestamps}));
}


sub get_previous_revert_date
{
    my $revert_to_date = $EMPTY;
    my @timestamps = ();

    my $backup_dir = $DAISY_DIR_DEF . $DAISY_BACKUP_DIR;
    get_tel_set_dates($backup_dir, \@timestamps);

    # now sort them and select the last one in the list
    if (@timestamps) {
	my @timestamps_sorted = sort(@timestamps);
	$revert_to_date = $timestamps_sorted[-1];
    }

    return($revert_to_date);
}


sub get_specific_revert_date
{
    my ($revert_to_date) = @_;

    my @timestamps = ();

    my $backup_dir = $DAISY_DIR_DEF . $DAISY_BACKUP_DIR;
    get_tel_set_dates($backup_dir, \@timestamps);

    if (! grep {$revert_to_date} @timestamps) {
	$revert_to_date = $EMPTY;
    }

    return($revert_to_date);
}



#################################################
#########   D O C U M E N T A T I O N   #########
#################################################


__END__

=pod

=head2 NAME

I<edir_revert> - script to revert to previous edir files


=head2 VERSION

This documentation refers to version: $Revision: 1.2 $



=head2 OPTIONS

=over 4

=item B<--version>

Output the version number of the script and exit.

=item B<--help>

Output a short help message and exit.

=item B<--verbose>

Modifier: report extra information.

=item B<--date=s>

Revert to the set of "tel files" specified by date.
The date format is "YYYYMMDDHHMMSS".

=item B<--previous>

Revert to the most previous set of "tel files".


=back


=head2 DESCRIPTION

The I<edir_revert.pl> script may be used to revert to a previous
set of edir files if the newly generated files have an issue.

Upon execution, the script first verifies several aspects of the
environment that are required for successful completion.
If any of these checks fails, logs an error, and exits with
an appropriate exit status.
For some cases, it also outputs a usage message,

=over 4

=item 1.

The script checks that only one or none of B<--previous> and
B<--date=s> are specified on the command line.

=item 2.

effective user id check: the script must be running under the root account or
under sudo.

=item 3.

The script then looks for a set of "tel files" to revert to
as specified on the command line or according to defaults.
If neither the B<--previous> nor the B<--date=s> option are specified, or
if only B<--previous> is specified,
the scripts checks the F</d/daisy/backups> directory to see if there
is a previous set of "tel files" to revert to.
If not, an error is logged and the script exits.
If only B<--date=s> is specified, the script looks for a set
of "tel files" corresponding to that date to revert to.
If not, an error is logged and the script exits.

=item 4.

Now that a set of "tel files" to revert to has been identified,
the script attempts to revert to that set.
If any errors occur, an error is logged and the script exists.

=back


=head2 Command Line Options

Only one of B<--previous> or B<--date=s> may appear on the command line
at the same time.
If neither appear, the default is as if B<--previous> was specified.

The date format for the B<--date=s> option is "YYYYMMDDHHMMSS".


=head1 EXAMPLES

To revert to the previous set of "tel files", simply enter:

$ sudo perl /d/daisy/bin/edir_revert.pl


=head1 FILES

=over 4

=item F</d/daisy>

This is the Daisy default database directory and it must exist.

=item F</d/daisy/backups>

The directory where previous sets of "tel files" are stored.

=item F</d/daisy/log/edir_revert.log>

Log file.

=back


=head2 DIAGNOSTICS

=over 4

=item Exit status 0 ($EXIT_OK)

Successful completion.

=item Exit status 1 ($EXIT_COMMAND_LINE)

In general, there was an issue with the syntax of the command line.

=item Exit status 2 ($EXIT_MUST_BE_ROOT)

The user must be root or running under "sudo".

=item Exit status 10 ($EXIT_REVERT_DATE_FORMAT)

The specified revert date must be of format: YYYYMMDDHHMMSS

=item Exit status 11 ($EXIT_REVERT_UNAVAILABLE)

The specified date to revert to did not exist.

=item Exit status 12 ($EXIT_REVERT_ERROR)

There was an error restoring the selected tel set back
to a daisy database dir.

=back


=head1 SEE ALSO

I<edir_installbase.pl>, I<edir_update.pl>, I<GetDeltas(1)>, I<maketel(1)>


=cut
