#!/usr/bin/perl
# 
# $Revision: 1.39 $
# Copyright 2011 Teleflora.
#
# The script constitutes the postpatch script for the process of
# applying patches to Daisy systems and for converting a Daisy
# system from the Global East payment system to Elavon.
#
# Called from applypatch.pl with arguments of:
#
#	ARGV[0] = path to current daisy db dir
#	ARGV[1] = time stamp
#	ARGV[2] = list of patch files (optional)
#

use strict;
use warnings;
use Getopt::Long;
use POSIX;
use File::Basename;

my $CVS_REVISION = '$Revision: 1.39 $';
my $PROGNAME = basename($0);

#
# Command line options
#
my $HELP = 0;
my $VERSION = 0;
my $DAISY = 0;
my $DAISY_DIR = '/d/daisy';
my $EDIRUPDATE = 1;
my $INSTALL_FTD_FILES = 0;
my $DEBUG = 0;

#
# Alternate os and root for testing.
#
my $ALTROOT = "";
my $ALTOS = "";

#
# Global variables
#
my $OS = "";
my $POSDIR = "";
my $LOGFILENAME = "log/RTI-Patches.log";
my @NU_BINARIES = ();
my @NU_DATAFILES = ();
my $TIMESTAMP = "";
my $DAISY_FTD_SUBS_FILE = "daisy-ftd-subs.txt";
my $APPLYPATCH_CONFIG_FILE = "applypatch.config";
my @CONFIG_QA_SHOPCODE = ();

#
# Constants
#

# Exit status values
my $EXIT_OK = 0;
my $EXIT_ERROR = -1;
my $EXIT_COMMAND_LINE = 1;
my $EXIT_MUST_BE_ROOT = 2;
my $EXIT_PATCH_VALIDATION = 3;
my $EXIT_POSTPATCH_CLEANUP = 8;
my $EXIT_STOP_DAISY = 9;
my $EXIT_START_DAISY = 10;
my $EXIT_PATCH_FAILED = 11;
my $EXIT_POSTPATCH = 12;


GetOptions(
	"help" => \$HELP,
	"version" => \$VERSION,
	"daisy" => \$DAISY,
	"daisydir:s" => \$DAISY_DIR,
	"edirupdate" => \$EDIRUPDATE,
	"install-ftd-files" => \$INSTALL_FTD_FILES,
	"debug" => \$DEBUG,
) || die "Error: invalid command line option, exiting...\n";


@NU_BINARIES = qw(
	edir
);

@NU_DATAFILES = qw(
);


# --version
if ($VERSION) {
	print("$PROGNAME $CVS_REVISION\n");
	exit(0);
}

# --help
if ($HELP) {
	print("$PROGNAME $CVS_REVISION\n");
	usage();
	exit(0);
}


unless (defined($ARGV[0])) {
	print("ARGV[0] not defined... expecting path to daisy db dir... exiting\n");
	exit(1);
}
if ($ARGV[0] eq "") {
	print("ARGV[0] null... expecting path to daisy db dir... exiting\n");
	exit(1);
}

#
# If daisy not specificed on the command line, then verify the existence
# of the default daisy database.
#
if ($DAISY == 0) {
	unless (-d $DAISY_DIR) {
		print("Directory $DAISY_DIR not found... exiting.");
		exit(2);
	}
	$DAISY = 1;
}

$POSDIR = $DAISY_DIR;

# now we can specify the location of the logfile
my $LOGFILE = "$POSDIR/$LOGFILENAME";

unless (defined($ARGV[1])) {
	print("ARGV[1] not defined... expecting time stamp of patch... exiting\n");
	exit(1);
}
if ($ARGV[1] eq "") {
	print("ARGV[1] null... expecting time stamp of patch... exiting\n");
	exit(1);
}

$TIMESTAMP = $ARGV[1];


#
# -->
# --> Read the config file
# -->
#
read_configfile("$DAISY_DIR/config/$APPLYPATCH_CONFIG_FILE");


sub main
{
    my $exit_status = 0;

    loginfo("----------------------------------------------------------------");
    loginfo("Begin post patch");
    loginfo("----------------------------------------------------------------");

    if ($EDIRUPDATE) {
	$exit_status = postpatch_edir_update($ARGV[0]);
    }


    #
    # Remove the postpatch.pl script we are currently running.
    #
    remove_postpatch_script($ARGV[0]);

    #
    # Remove the file of Daisy FTD subs if present
    #
    remove_daisy_ftd_subs_file($ARGV[0]);

    loginfo("----------------------------------------------------------------");
    loginfo("Completed post patch");
    loginfo("----------------------------------------------------------------");

    if ($exit_status == 0) {
	my $daisy_dir = $ARGV[0];
	if (-e "$daisy_dir/logevent") {
	    system("$daisy_dir/logevent nEventSystemUpdate 0 \"\" SYS postpatch.pl \"Your florist directory has been updated\"");
	}
    }

    return($exit_status);
}


#######
#### Functions
#######


sub usage
{
    print("Usage:\n");
    print("$PROGNAME --version\n");
    print("$PROGNAME --help\n");
    print("$PROGNAME --daisy\n");
    print("$PROGNAME --daisydir=path\n");
    print("$PROGNAME --edirupdate\n");
    print("\n");
}


#
# Read the config file located in default daisy db dir: /d/daisy/config
#
# Returns
#   0 on success
#   1 on error
#
sub read_configfile
{
    my $config_file = $_[0];

    unless ($config_file) {
	return(0);
    }
    if ($config_file eq "") {
	return(0);
    }
    unless (-e $config_file) {
	loginfo("Config File does not exist: $config_file");
	return(0);
    }

    loginfo("Reading config file: $config_file");

    unless (open(CONFIG, '<', $config_file)) {
	logerror("Can't open config file: $config_file");
	return(1);
    }

    while(<CONFIG>) {

	# qa-shopcode=nnnnnn00
	# Multiple instances of this line are allowed.
	if (/^(\s*)(qa-shopcode)(\s*)(=)(\s*)([[:print:]]+)$/i) {
		push(@CONFIG_QA_SHOPCODE, $6);
	}
    }

    close(CONFIG);

    return(0);
}


sub remove_postpatch_script
{
	my $postpatch_script_path = "$_[0]/bin/postpatch.pl";
	my $rc = 0;

	loginfo("Removing postpatch script: $postpatch_script_path");
	$rc = mysystem("rm -f $postpatch_script_path");
	if ($rc != 0) {
		logerror("Remove of postpatch script returns non-zero exit status: $rc");
	}

	return($rc);
}


sub remove_daisy_ftd_subs_file
{
    my $daisy_ftd_subs_path = "$_[0]/$DAISY_FTD_SUBS_FILE";
    my $rc = 0;

    if (-f $daisy_ftd_subs_path) {
	loginfo("Removing Daisy FTD subs file: $daisy_ftd_subs_path");
	$rc = mysystem("rm -f $daisy_ftd_subs_path");
	if ($rc != 0) {
	    logerror("Can't remove Daisy FTD subs file: $daisy_ftd_subs_path");
	}
    }

    return($rc);
}


#
# Get the system run level.
#
# Returns 0-6 on success, -1 on error
#
sub get_runlevel
{
    my $runlevel = -1;

    #
    # Output of runlevel is:
    #	(digit|letter) space digit newline
    # where the right most digit is the current runlevel
    #
    my $runlevelcmd = '/sbin/runlevel';
    if (open(RL, "$runlevelcmd |")) {
	while (<RL>) {
	    $runlevel = $_;
	    chomp($runlevel);
	    $runlevel =~ s/^. (\d)$/$1/;
	}
	close(RL);
    }
    else {
	showerror("Unable to use the run level command: $runlevelcmd");
    }

    return($runlevel);
}


#
# Set the system run level to value between 0 and 6.
#
# Returns 0 on success, -1 on error.
#
sub set_runlevel
{
    my ($new_runlevel) = @_;
    my $rc = 0;

    if ($new_runlevel < 0 || $new_runlevel > 6) {
	showerror("Can't happen: new system runlevel: $new_runlevel");
	return(-1);
    }

    system("/sbin/telinit $new_runlevel");
    if ($? != 0) {
	$rc = -1;
    }

    # The "init" man page mentions that "init" waits 5 seconds
    # between each of two kills, and testing reveals we need
    # to wait a bit for the runlevel to change.
    sleep(30);

    return($rc);
}


#
# Stop all Daisy processes.
#
# Return 0 on success, -1 on error
#
sub stop_daisy
{
        loginfo("Changing to system runlevel 4");
	# try to change the system runlevel
        unless (set_runlevel(4) == 0) {
            logerror("Can't change to system runlevel 4");
            return(-1);
        }

        # verify system runlevel is what we expect
        unless (get_runlevel() == 4) {
            logerror("Can't verify system runlevel is 4");
            return(-1);
        }
        loginfo("System runlevel changed to 4");

	# now kill off all daisy processes
	my $killemall_cmd = "/d/daisy/bin/killemall.pl";
        if (-f $killemall_cmd) {
            loginfo("Killing all Daisy processes");
            system("$killemall_cmd");
	    if ($? != 0) {
                showerror("Unable to kill all Daisy processes");
                return(-1);
            }
            loginfo("All Daisy processes killed");
        }
	else {
	    logerror("Expecting script to exist: $killemall_cmd");
	    return(-1);
	}

	return(0);
}


#
# Start Daisy.
#
# Returns 0 on success, -1 on error
#
sub start_daisy()
{
    my $rc = 0;

    if (set_runlevel(3) != 0) {
	logerror("Can't set runlevel to 3");
	return(-1);
    }

    if (get_runlevel() != 3) {
	logerror("Unable to return to runlevel 3");
	return(-1);
    }

    loginfo("Daisy restarted and system back to run level 3");

    return(0);
}


#
# Read shop code from Dove control file in default daisy database dir.
#
# Returns:
#   shop code as 8 digit string if found else empty string
#
sub get_shopcode
{
    my $daisy_db_dir = $_[0];
    my $shopcode = "";

    if (-d $daisy_db_dir) {
	my $dovectrl_file = "$daisy_db_dir/dovectrl.pos";

	unless (-f $dovectrl_file) {
	    logerror("Dove control file not found: $dovectrl_file");
	    return "";
	}

	unless (open(DF, $dovectrl_file)) {
	    logerror("Can't open Dove control file: $dovectrl_file");
	    return "";
	}

	my $buffer = "";
	my $bufsize = 50;
	my $rc = sysread(DF, $buffer, $bufsize);

	unless (defined($rc)) {
	    logerror("Error reading Dove control file: $!\n");
	    return "";
	}
	if ($rc == 0) {
	    logerror("Could not read Dove control file: $dovectrl_file");
	    return "";
	}
	if ($rc != $bufsize) {
	    logerror("Short read from Dove control file: $dovectrl_file");
	    logerror("Expected read of $bufsize but got $rc");
	    return "";
	}

	close(DF);

	$shopcode = substr($buffer, 30, 8);
    }
}


#
# Look up the shop code from "the list", ie list of shops that have purchased
# the FTD files.
#
# Returns:
#   1 if shop code is on the list, else 0
#
sub lookup_shopcode
{
    my $daisy_db_dir = $_[0];
    my $shopcode = $_[1];
    my $rc = 0;
    my $daisy_ftd_subs_path = "$daisy_db_dir/$DAISY_FTD_SUBS_FILE";

    #
    # Check to see if the shopcode is one of the QA shopcodes that
    # may be specified in the config file. 
    #
    if (@CONFIG_QA_SHOPCODE) {
	foreach (@CONFIG_QA_SHOPCODE) {
	    if (/$shopcode/) {
		loginfo("Shopcode matched QA shopcode from config file: $shopcode");
		loginfo("Allowing FTD subscription");
		return(1);
	    }
	}
    }

    unless (-f $daisy_ftd_subs_path) {
	logerror("Daisy FTD subscribers file not found: $daisy_ftd_subs_path");
	loginfo("Allowing FTD subscription");
	return(1);
    } 

    unless (open(SF, '<', $daisy_ftd_subs_path)) {
	logerror("Can't open Daisy FTD subscribers file: $daisy_ftd_subs_path");
	loginfo("Allowing FTD subscription");
	return(1);
    }

    while (<SF>) {

        # skip comments or blank lines
        next if ( (/^(\s*)#/) || (/^(\s*)$/) );

	# remove newline
	chomp;

	# remove leading and trailing SPACE chars
	my $subscriber = trim($_);

	# finally do the comparison
	if ($subscriber eq $shopcode) {
	    loginfo("Shopcode matched shopcode from subscriber file: $shopcode");
	    loginfo("Allowing FTD subscription");
	    $rc = 1;
	    last;
	}
    }

    close(SF);

    return($rc);
}


#
# Find the shop code, and lookup the shop code in the "paid" list;
# if the shop code is not on the paid list, then remove the FTD files
# from the backup (if they exist) and from the current daisy db dir.
#
# Returns
#   0 on success, non-0 on error
#
sub resolve_ftd_files
{
    my $daisy_db_dir = $_[0];
    my $rc = 0;

    my $DEF_DAISY_DB_DIR = "/d/daisy";

    #
    # If the command line option specified, leave the FTD files
    # installed without any further checks.
    #
    if ($INSTALL_FTD_FILES) {
	loginfo("FTD files installation forced with \"--install-ftd-files\" option");
	return(0);
    }
    else {
	my @ftd_backup_files = glob("$daisy_db_dir/backups/*.ftd.prepatch.*");
	unless (@ftd_backup_files) {
	    loginfo("Non-FTD subscriber, removing new FTD files");
	    $rc = mysystem("rm -f $daisy_db_dir/*.ftd");
	    if ($rc != 0) {
		logerror("Removal of FTD files failed ($rc)");
		return(1);
	    }
	}
	return(0);
    }


    my $shopcode = get_shopcode($DEF_DAISY_DB_DIR);
    if ($shopcode eq "") {
	logerror("Could not get shop code");
	return(1);
    }

    #
    # At this point, there may be old FTD files in the backup dir and
    # there may be new FTD files in the daisy db dir.
    #
    # Now, the shopcode must be on "the list" or both any old FTD files and
    # any new FTD files are removed.
    #
    # If the shopcode is not on "the list", then remove the new FTD files from
    # the daisy db dir and the old FTD files from the backup dir.

    unless (lookup_shopcode($daisy_db_dir, $shopcode)) {

	loginfo("Shop with shop code $shopcode is a non-FTD subscriber");
	loginfo("Removing new FTD files");

	$rc = mysystem("rm -f $daisy_db_dir/*.ftd");
	if ($rc != 0) {
	    logerror("Removal of new FTD files failed ($rc)");
	}

	loginfo("Removing old FTD files");

	my @ftd_backup_files = glob("$daisy_db_dir/backups/*.ftd.prepatch.*");
	foreach (@ftd_backup_files) {
	    $rc = mysystem("rm -f $_");
	}
    }

    return($rc);
}


sub resolve_hsp_files
{
	my $daisy_db_dir = $_[0];
	my $rc = 0;

	my @hsp_backup_files = glob("$daisy_db_dir/backups/hnot*.hsp.prepatch.*");

	unless (@hsp_backup_files) {
		return($rc);
	}

	foreach my $hsp_backup_file (@hsp_backup_files) {

		# isolate the filename
		my $hsp_filename = basename($hsp_backup_file);

		# convert it back to original
		if ($hsp_filename =~ /(.*)\.prepatch\..*$/) {
			$hsp_filename = $1;
		}

		loginfo("Restoring pervious hospital note file: $hsp_filename");
		$rc = mysystem("cp $hsp_backup_file $daisy_db_dir/$hsp_filename");
		if ($rc != 0) {
			logerror("Restoration of hospital note file ($hsp_filename) failed ($rc)");
			last;
		}
	}

	return($rc);
}


#
# resolve_binaries
#
# foreach new binary file
#	form a platform specific file name
#	if there is platform specific file
#		copy the original binary to the backup directory
#		mv the platform specific file to the original file
#
#
# Return 0 on success, non-zero otherwise
#
sub resolve_binaries
{
    my $daisy_dir = $_[0];
    my $rc = 0;

    determine_os();

    #
    # "RHWS5" is really "Redhat Enterprise Linux Client" and for the
    # purposes of chosing the correct binary, we must treat "Client"
    # systems like "Server" systems.
    #
    if ($OS eq "RHWS5") {
	$OS = "RHEL5";
    }

    if ( ($OS ne 'RHEL5') && ($OS ne 'RHEL6') && ($OS ne 'RHEL7') ) {
	logerror("Unsupported platform: $OS");
	return(1);
    }

    #
    # FIXME: remove non-applicable platform binaries 
    #

    foreach my $nu_binary (@NU_BINARIES) {

	my $platform_binary = $nu_binary . "-" . $OS;

	if (-f "$daisy_dir/$platform_binary") {
	    loginfo("Installing new binary: $nu_binary for platform: $OS");
	    my $tmp_rc = mysystem("cp $daisy_dir/$nu_binary $daisy_dir/backups");
	    if ($tmp_rc != 0) {
		logerror("Backup of $nu_binary failed ($rc)");
		$rc = 1;
	    }
	    else {
		$tmp_rc = mysystem("mv $daisy_dir/$platform_binary $daisy_dir/$nu_binary");
		if ($tmp_rc != 0) {
		    logerror("Installation of $nu_binary failed ($rc)");
		    $rc = 1;
		}

		system("chown tfsupport:daisy $daisy_dir/$nu_binary");
		system("chmod 550 $daisy_dir/$nu_binary");
	    }
	}
	else {
	    logerror("New binary for platform ($OS) not available in patch: $nu_binary");
	    $rc = 1;
	}
    }

    return($rc);
}		


sub resolve_data_files
{
    my $daisy_db_dir = $_[0];
    my $rc = 0;

    # nothing to do

    return($rc);
}


#
# (1) If there are no FTD files in the backup dir, then the assumption is
#     that this daisy system does not subscribe to FTD.  So, remove
#     the FTD files from the current daisy db dir.
#
# (2) If there are "hospital note files" in the backup dir, that means
#     there were some in the daisy db dir before the patch was applied...
#     so the new empty files need to be removed from the current daisy
#     db dir and the old ones copied from the backup dir to the current
#     daisy db dir.
#
# (3) If there were new or updated binaries, remove the old ones, and
#     install the new ones.
#
sub postpatch_edir_update
{
	my $daisy_db_dir = $_[0];
	my $rc = 0;

	if (resolve_ftd_files($daisy_db_dir)) {
		$rc++;
	}

	if (resolve_hsp_files($daisy_db_dir)) {
		$rc++;
	}

	if (resolve_binaries($daisy_db_dir) != 0) {
		$rc++;
	}

	return($rc);
}


# trim whitespace from start and end of string
sub trim
{
	my $string = $_[0];

	$string =~ s/^\s+//;
	$string =~ s/\s+$//;

	return $string;
}


sub fill_left
{
    my $string = $_[0];
    my $fill_char = $_[1];
    my $str_max = $_[2];

    my $str_len = length($string);

    if ($str_len < $str_max) {
	my $fill_size = $str_max - $str_len;
	for (my $i = 0; $i < $fill_size; $i++) {
	    $string = $fill_char . $string;
	}
    }

    return($string);
}


sub backout_patch
{
    my $daisy_dir = $_[0];

    my $os = determine_os();

    foreach my $thisfile (@NU_BINARIES) {

	# since this is a binary, when the file was backed up,
	# a platform suffix was added to it so we must add the
	# same suffix to the name.
	my $platform_binary = $thisfile . "-" . $os;

	my $src_file = "$daisy_dir/backups/$platform_binary.prepatch.$TIMESTAMP";
	my $dst_file = "$daisy_dir/$thisfile";

	if (! -f $src_file) {
	    loginfo("Warning: can't back out patched file: $src_file does not exist");
	    next;
	}

	loginfo("Backing out $src_file -> $dst_file");
	mysystem("cp -f $src_file $dst_file 2> /dev/null");
	if ($? != 0) {
	    # We're pretty much hosed at this point.
	    logerror("Backout of $src_file $dst_file Failed ($?)");
	}
    }

    foreach my $thisfile (@NU_DATAFILES) {

	my $src_file = "$daisy_dir/backups/$thisfile.prepatch.$TIMESTAMP";
	my $dst_file = "$daisy_dir/$thisfile";

	if (! -f $src_file) {
	    loginfo("Warning: can't back out patched file: $src_file does not exist");
	    next;
	}

	loginfo("Backing out $src_file -> $dst_file");
	mysystem("cp -f $src_file $dst_file 2> /dev/null");
	if ($? != 0) {
	    # We're pretty much hosed at this point.
	    logerror("Backout of $src_file $dst_file Failed ($?)");
	}
    }

    # fix up the perms after copying files back
    my $perms_cmd = "$daisy_dir/bin/dsyperms.pl";
    if (-e $perms_cmd) {
	loginfo("After backing out patch, setting perms on: $daisy_dir");
	mysystem("$perms_cmd $daisy_dir");
	if ($? != 0) {
	    logerror("Error occurred running script: $perms_cmd");
	}
    }
    else {
	logerror("Should not happen: script does not exist: $perms_cmd");
    }
}


sub determine_os
{
	$OS = "";

	# We could also use /usr/bin/lsb-release
	open(FILE, '<', "/etc/redhat-release");
	while(<FILE>) {

		# ES 7
		# Redhat Enterprise Linux Server 7
		if( (/(release)(\s+)(7)/)
		||  (/(CentOS)([[:print:]]+)(\s)(7)/) ) {
			$OS = "RHEL7";
			last;
		}

		# ES 6
		# Redhat Enterprise Linux Server 6
		if( (/(release)(\s+)(6)/)
		||  (/(CentOS)([[:print:]]+)(\s)(6)/) ) {
			$OS = "RHEL6";
			last;
		}

		# ES 5
		# Redhat Enterprise Linux Server 5
		if( (/(release)(\s+)(5)/)
		||  (/(CentOS)([[:print:]]+)(\s)(5)/) ) {
			$OS = "RHEL5";
			last;
		}

		# EL 4
		# Redhat Enterprise Linux Server 4
		if( (/(release)(\s+)(4)/) 
		||  (/(CentOS)([[:print:]]+)(\s)(4)/) ) {
			$OS = "RHEL4";
			last;
		}
	}
	close(FILE);

	return($OS);
}


#
# Subroutine to run system commands.  This logs an entry.
#
sub mysystem
{
    my $command = $_[0];
    my $returnval = "";

    if ($DEBUG) {
	system("$command >> $LOGFILE 2>> $LOGFILE");
    }
    else {
	system($command);
    }
    $returnval = WEXITSTATUS($?);
    if ($DEBUG) {
        loginfo("$command Returned: $returnval");
    }

    return ($returnval)
}



#
# Log an item to the logfile.
#
sub loginfo
{
        my $message = $_[0];
        my $timestamp = "";

        chomp ($message);
        $timestamp = strftime("%Y-%m-%d %H:%M:%S", localtime());

        open LOGFILE, '>>', "$LOGFILE";
                print LOGFILE "$timestamp <I>: $message\n";
        close LOGFILE;


        return "";
}

#
# Log an error to the logfile.
#
sub logerror
{
        my $message = $_[0];
        my $timestamp = "";

        chomp ($message);
        $timestamp = strftime("%Y-%m-%d %H:%M:%S", localtime());

        open LOGFILE, '>>', "$LOGFILE";
                print LOGFILE "$timestamp <E>: $message\n";
        close LOGFILE;


        return "";
}


exit(main());


__END__

=pod

=head1 NAME

postpatch.pl - apply custom changes after applying a patch


=head1 VERSION

This documenation refers to version: $Revision: 1.39 $


=head1 USAGE

postpatch.pl

postpatch.pl B<--version>

postpatch.pl B<--help>

postpatch.pl B<--daisy>

postpatch.pl B<--daisydir=path>

postpatch.pl B<--install-ftd-files>


=head1 OPTIONS

=over 4

=item B<--version>

Output the version number of the script and exit.

=item B<--help>

Output a short help message and exit.

=item B<--daisy>

If the directory of the default Daisy database directory "/d/daisy"
does not exist or the command line option "--daisy" is not specified,
the script outputs an error message to stdout and
exits with an exit status of 2.

=item B<--daisydir=path>

Change the value of the default Daisy database directory to that
specified as C<path>.

=item B<--install-ftd-files>

When performing a post patch procedure, if this command line option
is specified, the normal check for the shopcode of the system on the
list of subscribers to the Daisy FTD files will be skipped and the Daisy
FTD files will be installed.

=back


=head1 DESCRIPTION

The usual patch procedure for a Teleflora Daisy POS
involves first running the "applypatch.pl" script to apply a "patch",
i.e. a set of files and contained in a compressed tar archive.
One of the files in the "patch" is the "postpatch.pl" script.
It's behavior is customized for each "patch".
After the files from the patch are applied
(really just copied into place after saving any old instances
of files in the patch in the backup directory), then
this "postpatch.pl" script is run.
Both scripts are run on each of any existing Daisy database dirs.
Usually, the scripts are run from within the context of an
Altiris push, but it can be downloaded to a system and
kicked off manually.


=head1 FILES

=over 4

=item B</d/daisy/config/applypatch.config>

The configuration file for the apply patch process.
Currently, the only variable that may be defined in the config file
is "qa-shopcode=nnnnnnnn" where "nnnnnnnn" represents a shopcode.
There may be zero or more of these lines in the config file.

=item B<daisy-ftd-subs.txt>

This file contains the list of Daisy FTD subscriber shopcodes.
When using the "applypatch" technology for distributing a new
set of electronic directory files, it is one of the files in the
the patch file.

If the shopcode of the system that the patch is running on
does NOT appear in this file, then a message will be logged and
the FTD files will NOT be installed.

However, if the shopcode of the system is defined in the
config file via the "qa-shopcode" variable,
then the check for appropriate shopcode will pass and
the FTD files will be installed.

=item B</d/daisy/log/RTI-Patches.log>

The log file that all log messages are written to.

=item B</d/daisy/backups>

Each Daisy database directory has a directory named "backups" which
is where old instances of files in the "patch" are copied before
being replaced with the new instances.

=back


=head1 DIAGNOSTICS

=over 4

=item Exit status 0

Successful completion or when the "--version" and "--help"
command line options are specified.

=item Exit status 1

If the path to the Daisy database directory to "post patch" is NOT
specified as the first argument on the command line.
If the time stamp is NOT
specified as the second argument on the command line.

=item Exit status 2

If the default Daisy database directory does not exist.

=item Exit status 9

The conversion failed because Daisy could not be stopped.

=item Exit status 10

The conversion failed because Daisy could not be restarted.

=item Exit status 11

If the conversion to Elavon failed.

=back


=head1 SEE ALSO


=cut
